/**
 * @file dmxset.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ----------------------------------------------------------------------------
 * (c) Florian Schmitt / 2012-2013
 * License GPLv3+: GNU GPL Version 3 
 * or higher <http://gnu.org/licenses/gpl.html>
 * This code comes with ABSOLUTELY NO WARRANTY. This is free software, and 
 * you are welcome to redistribute it under certain conditions.
 * ----------------------------------------------------------------------------
 * A simple program to set a channel's value on a lindmxd 
**/
#include "dmxset.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>


#define BUFLEN 1024
int global_sock = 0;
int fadebuf[512];



/**
 * kill_lineend()
 * Kills the line end. The incoming string format does not matter: 
 * '\r', '\n', '\r\n' and '\n\r' is accepted.
 * @param str    String to modify
 * @param buflen buffer length, maximum place to go to
 * @return string length
**/
int kill_lineend(char *str, int buflen)
{
	int n = 0;
	/* look for the line/string end */
	while( n<buflen && (str[n] != '\r') && (str[n] != '\n') && (str[n] != '\0') ) ++n;
	/* place a \0 at the end */
	str[n] = '\0';
	return (n-1);
}





/**
 * dmxset()
 * Sets a dmx channel via tcp/ip
**/
int dmxset(int channel, int value, const char *remote, int port)
{

	/* client connection */
	int sock;
	struct hostent *host;
	struct sockaddr_in addr;
	char buffer[BUFLEN];

	if ( (host = gethostbyname(remote)) == NULL) 
	{
		return DMXSET_ERR_HOSTNOTFOUND;
	}
	
	sock = socket(AF_INET, SOCK_STREAM, 0);
	addr.sin_family      = AF_INET;
	addr.sin_addr.s_addr = *((unsigned long *) host->h_addr_list[0]);
	addr.sin_port        = htons(port);
	
	/* try to connect */
	if(sock == 0) return DMXSET_ERR_CONNECTION;
	if( connect(sock, (struct sockaddr *)&addr, sizeof(addr)) != 0 )
	{
		
		return DMXSET_ERR_CONNECTION;
	}

	/* send data */
	sprintf(buffer, "set %d %d\r\n", channel, value);
	if( write(sock, buffer, strlen(buffer)) <= 0 ) 
	{
		close(sock);
		return DMXSET_ERR_SEND;
	}

	/* receive answer */
	if( read(sock, buffer, BUFLEN) <= 0 ) 
	{
		close(sock);
		return DMXSET_ERR_RECV;
	}

	/* check answer */
	kill_lineend(buffer, BUFLEN);
	if( strcmp(buffer, "ok") != 0 )
	{
		close(sock);
		return DMXSET_ERR_COMMAND;
	}

	/* close connection */
	close(sock);
	return DMXSET_ERR_NOERROR;
}




/**
 * dmxset_global_init()
 * Initiates dmxset for global use with an consisting connection
**/
int dmxset_global_init(const char *remote, int port)
{
	/* client connection */
	int sock;
	struct hostent *host;
	struct sockaddr_in addr;

	if ( (host = gethostbyname(remote)) == NULL) 
	{
		return DMXSET_ERR_HOSTNOTFOUND;
	}
	
	sock = socket(AF_INET, SOCK_STREAM, 0);
	addr.sin_family      = AF_INET;
	addr.sin_addr.s_addr = *((unsigned long *) host->h_addr_list[0]);
	addr.sin_port        = htons(port);
	
	/* try to connect */
	if(sock == 0) return DMXSET_ERR_CONNECTION;
	if( connect(sock, (struct sockaddr *)&addr, sizeof(addr)) != 0 )
	{
		return DMXSET_ERR_CONNECTION;
	}
	global_sock = sock;
	return DMXSET_ERR_NOERROR;
}




/**
 * dmxset_global()
 * Sets a dmx channel via tcp/ip on the global conneciton
**/
int dmxset_global(int channel, int value)
{
	int sock;
	char buffer[BUFLEN];

	if(global_sock == 0) return DMXSET_ERR_CONNECTION;
	sock = global_sock;

	/* send data */
	sprintf(buffer, "set %d %d\r\n", channel, value);
	if( write(sock, buffer, strlen(buffer)) <= 0 ) 
	{
		close(sock);
		return DMXSET_ERR_SEND;
	}

	/* receive answer */
	if( read(sock, buffer, BUFLEN) <= 0 ) 
	{
		close(sock);
		return DMXSET_ERR_RECV;
	}

	/* check answer */
	kill_lineend(buffer, BUFLEN);
	if( strcmp(buffer, "ok") != 0 )
	{
		close(sock);
		return DMXSET_ERR_COMMAND;
	}
	return DMXSET_ERR_NOERROR;
}



/**
 * dmxset_global_close()
 * Closes global connection
**/
void dmxset_global_close()
{
	if(global_sock == 0) return;
	close(global_sock);
	global_sock = 0;
}



/**
 * dmxset_global_loadcue()
 * Loads an dmx cue
 * @param list     List of dmxpairs to set channels to
 * @param ignore   List of channels to ignore (e.g. from programmer), ignored if null
 * @return See dmxset_global()
**/
int dmxset_global_loadcue(dmxlist_t *list, dmxlist_t *ignore)
{
	int ret, tmp;
	dmxpair_t *p;
	/* set no-error */
	ret = DMXSET_ERR_NOERROR;
	/* iterate list */
	p = list->first;
	while( p != 0)
	{
		if( (ignore == 0) || (dmxcue_get_channel_obj(p->channel, ignore) == 0) )
		{
			tmp = dmxset_global((p->channel+1), p->value);
			if(tmp != DMXSET_ERR_NOERROR) ret = tmp;
		}
		p = dmxcue_get_next(p);
	}
	return ret;
}



/**
 * dmxset_global_loadcue_faded()
 * Loads an dmx cue faded
 * @param list     List of dmxpairs to set channels to
 * @param ignore   List of channels to ignore (e.g. from programmer), ignored if null
 * @param fadepos  Position of the fade in promille, sync on 1000
 * @return See dmxset_global()
**/
int dmxset_global_loadcue_faded(dmxlist_t *list, dmxlist_t *ignore, int fadepos)
{
	int ret, tmp, faded;
	dmxpair_t *p;
	/* set no-error */
	ret = DMXSET_ERR_NOERROR;
	/* iterate list */
	p = list->first;
	while( p != 0)
	{
		if( (ignore == 0) || (dmxcue_get_channel_obj(p->channel, ignore) == 0) )
		{
			/* on finished fade, take value as fadebuf value */
			if(fadepos >= 1000) fadebuf[p->channel] = p->value;
			/* calculate faded value */
			faded = p->value*fadepos  +  fadebuf[p->channel]*(1000-fadepos);
			faded = faded / 1000;
			/* set channel */
			tmp = dmxset_global((p->channel+1), faded);
			/* on error: store error in return value */
			if(tmp != DMXSET_ERR_NOERROR) ret = tmp;
		}
		p = dmxcue_get_next(p);
	}
	return ret;
}
