/**
 * @file modules/dmx4all_tcp.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ----------------------------------------------------------------------------
 * Implements dmx4all network interface driver
 * ----------------------------------------------------------------------------
 * (c) Florian Schmitt / 2012-2013
 * License GPLv3+: GNU GPL Version 3
 * or higher <http://gnu.org/licenses/gpl.html>
 * This code comes with ABSOLUTELY NO WARRANTY. This is free software, and 
 * you are welcome to redistribute it under certain conditions.
 * ----------------------------------------------------------------------------
 * This module is sponsored by zonezero GmbH <http://www.zonezero.de>
 * ----------------------------------------------------------------------------
**/
#include "dmx4all_tcp.h"
#include <syslog.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define BUFLEN 80

/* globals */
int maxchn       = 0;
const char *host = 0;
int port         = 0;
int sock         = 0;



/**
 * dmx4all_tcp_connect()
 * Reconnects to interface
 * @return 1 on success, 0 on failure
**/
int dmx4all_tcp_connect()
{
	char buffer[BUFLEN];
	struct hostent *host_ent;
	struct sockaddr_in addr;

	/* check settings */
	if(host == NULL)
	{
		syslog(LOG_ERR, "Illegal host value for networking interface.");
		return 0;
	}
	if(port == 0)
	{
		syslog(LOG_ERR, "Illegal port value for networking interface.");
		return 0;
	}

	/* resolve hostname */
	if ( (host_ent = gethostbyname(host)) == NULL) 
	{
		syslog(LOG_ERR, "Interface host'%s' can not be resolved", host);
		return 0;
	}

	/* create socket */
	sock = socket(AF_INET, SOCK_STREAM, 0);
	addr.sin_family       = AF_INET;
	addr.sin_addr.s_addr  = *((unsigned long *) host_ent->h_addr_list[0]);
	addr.sin_port         = htons(port);
	
	/* try to connect */
	if( connect(sock, (struct sockaddr *)&addr, sizeof(addr)) != 0 )
	{
		syslog(LOG_ERR, "Failed to connect to interface %s:%d", host, port);
		return 0;
	}

	/* send channel count data */
	sprintf(buffer, "N%03d\r\n", maxchn);
	if( write(sock, buffer, strlen(buffer)) <= 0 ) 
	{
		syslog(LOG_ERR, "Set max chn: Failed to send request to interface %s:%d", host, port);
		/* return failure */
		return 0;
	}

	/* receive answer */
	if( read(sock, buffer, BUFLEN) <= 0 ) 
	{
		syslog(LOG_ERR, "Set max chn: Failed to receive answer from %s:%d", host, port);
		return 0;
	}

	return 1;
}



/**
 * dmx4all_tcp_init()
 * Init DMX4ALL Micro USB Interface
 * @param maxchannel    The highest channel number created
 * @return 1 on success, 0 on failure
**/
int dmx4all_tcp_init(int maxchannel, const char *if_host, int if_port)
{
	/* settings */
	maxchn  = maxchannel;
	host    = if_host;
	port    = if_port;
	/* try connect */
	return dmx4all_tcp_connect();
}



/**
 * dmx4all_tcp_destroy()
 * Clean up interface
**/
void dmx4all_tcp_destroy()
{
	/* close connection */
	close(sock);
	sock = 0;
}



/**
 * dmx4all_tcp_reset_channels()
 * Resets all channels to zero
 * @param maxchannel    The highest channel number created
 * @return 1 on success, 0 on failure
**/
int dmx4all_tcp_reset_channels()
{
	return dmx4all_tcp_set_all( 0 );
}



/**
 * dmx4all_tcp_set_all()
 * Sets all channels to a given value
 * @param value    Value to set all channels to
 * @return 1 on success, 0 on failure
**/
int dmx4all_tcp_set_all(int value)
{
	int n, ret;
	/* loop through all channels */
	ret = 1;
	for(n=0; n<maxchn; n++)
	{
		if(!dmx4all_tcp_set(n, value)) ret = 0;
	}
	return ret;
}



/**
 * dmx4all_tcp_set()
 * Sets a channel to a given value
 * @param channel  Channel to set
 * @param value    Value to set to
 * @return 1 on success, 0 on failure
**/
int dmx4all_tcp_set(int channel, int value)
{
	char buffer[BUFLEN];

	/* send data */
	sprintf(buffer, "C%03dL%03d\r\n", channel, value);
	if( write(sock, buffer, strlen(buffer)) <= 0 ) 
	{
		syslog(LOG_ERR, "Failed to send request to interface %s:%d", host, port);
		/* try to reconnect */
		close(sock);
		sock = 0;
		dmx4all_tcp_connect();
		/* return failure */
		return 0;
	}

	/* receive answer */
	if( read(sock, buffer, BUFLEN) <= 0 ) 
	{
		syslog(LOG_ERR, "Failed to receive answer from %s:%d", host, port);
		return 0;
	}

	/* success */
	return 1;
}
